@@ -21,13 +21,6 @@ class PostgreSQLModel(data_algebra.db_model.DBModel):
         if not isinstance(identifier, str):
             raise TypeError("expected identifier to be a str")
         if self.identifier_quote in identifier:
-            raise ValueError('did not expect " in identifier')
+            # TODO: escape quotes
+            raise ValueError('did not expect ' + self.identifier_quote + ' in identifier')
         return self.identifier_quote + identifier.lower() + self.identifier_quote
-
-    def build_qualified_table_name(self, table_name, *, qualifiers=None):
-        qt = self.quote_identifier(table_name)
-        if qualifiers is None:
-            qualifiers = {}
-        if "schema" in qualifiers.keys():
-            qt = self.quote_identifier(qualifiers["schema"]) + "." + qt
-        return qt
