@@ -10,7 +10,9 @@ import data_algebra.eval_model
 
 # map from op-name to special SQL formatting code
 
-
+# Standard SQL code for checking isbad doesn't work in SQLlite, so
+# at least capture to is_bad, which appears to not be implemented
+# unless we call prepare connection
 def _sqlite_is_bad_expr(dbmodel, expression):
     return (
         "is_bad("
@@ -31,7 +33,6 @@ def _sqlite_lag_expr(dbmodel, expression):
     )
 
 
-# noinspection PyUnusedLocal
 def _sqlite_size_expr(dbmodel, expression):
     return "SUM(1)"
 
@@ -54,6 +55,17 @@ def _check_scalar_bad(x):
     return 0
 
 
+class MedianAgg:
+    def __init__(self):
+        self.collection = []
+
+    def step(self, value):
+        self.collection.append(value)
+
+    def finalize(self):
+        return numpy.median(self.collection)
+
+
 class SQLiteModel(data_algebra.db_model.DBModel):
     """A model of how SQL should be generated for SQLite"""
 
@@ -153,6 +165,9 @@ class SQLiteModel(data_algebra.db_model.DBModel):
                 conn.create_function(k, 1, f)
                 saw.add(k)
 
+        # https://docs.python.org/3/library/sqlite3.html
+        conn.create_aggregate('median', 1, MedianAgg)
+
     def quote_identifier(self, identifier):
         if not isinstance(identifier, str):
             raise TypeError("expected identifier to be a str")
