@@ -1,5 +1,5 @@
 __docformat__ = "restructuredtext"
-__version__ = "0.6.2"
+__version__ = "0.7.0"
 
 __doc__ = """
 `data_algebra`<https://github.com/WinVector/data_algebra> is a piped data wrangling system
@@ -18,8 +18,8 @@ the `rquery`<https://github.com/WinVector/rquery> and `rqdatatable`<https://gith
 import data_algebra.pandas_model
 
 # set up what pandas supplier we are using
-import pandas as pd
+import pandas
 
 default_data_model = data_algebra.pandas_model.PandasModel(
-    pd=pd, presentation_model_name="pd"
+    pd=pandas, presentation_model_name="pd"
 )
