@@ -1,3 +1,5 @@
+
+
 class Component:
     def __init__(self, item):
         self.id = item
@@ -68,31 +70,3 @@ def get_index_lists(partition_columns):
             index_lists[pi] = lst
         lst.append(i)
     return index_lists
-
-
-def partitioned_eval(fn, arg_columns, partition_columns):
-    """
-    Evaluate fn(fn_columns) on level-sets of partition columms.
-
-    :param fn: function with arity length(fn_columns) then returns lists/vectors of the common length of its arguments.
-    :param arg_columns: non-empty list of lists of length n
-    :param partition_columns: possibly empty list of lists of hashables/comparables of all length n
-    :return: list of length n of fn(fn_columns) evaluted on level-sets of partition_columns
-    """
-    if (partition_columns is None) or (len(partition_columns) <= 0):
-        return fn(*arg_columns)
-    arity = len(arg_columns)
-    res = [None] * len(arg_columns[0])
-    index_lists = get_index_lists(partition_columns)
-    for indexs in index_lists.values():
-        # slice out a level set
-        ni = len(indexs)
-        fn_cols_i = [
-            [arg_columns[j][indexs[i]] for i in range(ni)] for j in range(arity)
-        ]
-        # call function on slice
-        hi = fn(*fn_cols_i)
-        # copy back result
-        for i in range(ni):
-            res[indexs[i]] = hi[i]
-    return res
