@@ -267,7 +267,7 @@ class ViewRepresentation(OperatorPlatform, ABC):
 
     # Pandas realization
 
-    def eval_implementation(self, *, data_map, eval_env, data_model, narrow):
+    def eval_implementation(self, *, data_map, data_model, narrow):
         raise NotImplementedError("base method called")
 
     def check_constraints(self, data_model, *, strict=True):
@@ -298,11 +298,10 @@ class ViewRepresentation(OperatorPlatform, ABC):
                         "Table " + k + " has forbidden columns: " + str(excess)
                     )
 
-    def eval(self, data_map, *, eval_env=None, data_model=None, narrow=True):
+    def eval(self, data_map, *, data_model=None, narrow=True):
         """
          Evaluate operators with respect to Pandas data frames.
          :param data_map: map from table names to data frames
-         :param eval_env: environment to evaluate in
          :param data_model: adaptor to data dialect (Pandas for now)
          :param narrow logical, if True don't copy unexpected columns
          :return:
@@ -330,11 +329,11 @@ class ViewRepresentation(OperatorPlatform, ABC):
                 if not data_model.is_appropriate_data_instance(data_map[k]):
                     raise ValueError("data_map[" + k + "] was not a usable type")
         return self.eval_implementation(
-            data_map=data_map, eval_env=eval_env, data_model=data_model, narrow=narrow
+            data_map=data_map, data_model=data_model, narrow=narrow
         )
 
     # noinspection PyPep8Naming
-    def transform(self, X, *, eval_env=None, data_model=None, narrow=True):
+    def transform(self, X, *, data_model=None, narrow=True):
         if data_model is None:
             data_model = data_algebra.default_data_model
         if not isinstance(data_model, data_algebra.data_model.DataModel):
@@ -353,7 +352,6 @@ class ViewRepresentation(OperatorPlatform, ABC):
             data_map = {k: X}
             return self.eval(
                 data_map=data_map,
-                eval_env=eval_env,
                 data_model=data_model,
                 narrow=narrow,
             )
@@ -522,16 +520,10 @@ class ViewRepresentation(OperatorPlatform, ABC):
             {"expr": expr}, self, parse_env=parse_env
         )
         data_model = data_algebra.default_data_model
-        forbid = data_model.custom_function_map.keys()
 
         def r_walk_expr(op):
             if not isinstance(op, data_algebra.expr_rep.Expression):
                 return
-            if op.op in forbid:
-                raise ValueError("special functions such as "
-                                 + op.op
-                                 + " not allowed in select_rows.\n"
-                                 + " Work around: please use an extend to materialize this expression.")
             for oi in op.args:
                 r_walk_expr(oi)
 
@@ -725,11 +717,11 @@ class TableDescription(ViewRepresentation):
         raise an exception if the values are not consistent"""
         return {self.key: self}
 
-    def eval_implementation(self, *, data_map, eval_env, data_model, narrow):
+    def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
             raise ValueError("Expected data_model to not be None")
         return data_model.table_step(
-            op=self, data_map=data_map, eval_env=eval_env, narrow=narrow
+            op=self, data_map=data_map, narrow=narrow
         )
 
     def columns_used_from_sources(self, using=None):
@@ -796,6 +788,7 @@ class ExtendNode(ViewRepresentation):
         self, *, source, parsed_ops, partition_by=None, order_by=None, reverse=None
     ):
         windowed_situation = data_algebra.expr_rep.implies_windowed(parsed_ops)
+        ordered_windowed_situation = False
         self.ops = parsed_ops
         self.cols_used_in_calc = data_algebra.expr_rep.get_columns_used(parsed_ops)
         self.cols_produced_in_calc = [k for k in parsed_ops.keys()]
@@ -815,6 +808,7 @@ class ExtendNode(ViewRepresentation):
             order_by = [order_by]
         if len(order_by) > 0:
             windowed_situation = True
+            ordered_windowed_situation = True
         self.windowed_situation = windowed_situation
         self.order_by = order_by
         if reverse is None:
@@ -862,7 +856,7 @@ class ExtendNode(ViewRepresentation):
                         + "'"
                         + k
                         + "': '"
-                        + opk.to_pandas()
+                        + str(opk)
                         + "'"
                     )
                 if len(opk.args) > 1:
@@ -871,20 +865,38 @@ class ExtendNode(ViewRepresentation):
                         + "'"
                         + k
                         + "': '"
-                        + opk.to_pandas()
+                        + str(opk)
                         + "' term is too complex an expression"
                     )
                 if len(opk.args) > 0:
-                    value_name = opk.args[0].to_pandas()
-                    if value_name not in source.column_set:
-                        raise ValueError(
-                            "in windowed situations only simple operators are allowed, "
-                            + "'"
-                            + k
-                            + "': '"
-                            + opk.to_pandas()
-                            + "' term is too complex an expression"
-                        )
+                    if isinstance(opk.args[0], data_algebra.expr_rep.ColumnReference):
+                        value_name = opk.args[0].column_name
+                        if value_name not in source.column_set:
+                            raise ValueError(
+                                value_name + " not in source column set"
+                            )
+                    else:
+                        if not isinstance(opk.args[0], data_algebra.expr_rep.Value):
+                            raise ValueError(
+                                "in windowed situations only simple operators are allowed, "
+                                + "'"
+                                + k
+                                + "': '"
+                                + str(opk)
+                                + "' term is too complex an expression"
+                            )
+                if (ordered_windowed_situation and
+                        (opk.op in data_algebra.expr_rep.fn_names_that_contradict_ordered_windowed_situation)):
+                    raise ValueError(
+                        str(opk)
+                        + "' is not allowed in an ordered windowed situation"
+                    )
+                if ((not ordered_windowed_situation) and
+                        (opk.op in data_algebra.expr_rep.fn_names_that_imply_ordered_windowed_situation)):
+                    raise ValueError(
+                        str(opk)
+                        + "' is not allowed in not-ordered windowed situation"
+                    )
         ViewRepresentation.__init__(
             self, column_names=column_names, sources=[source], node_name="ExtendNode"
         )
@@ -927,11 +939,10 @@ class ExtendNode(ViewRepresentation):
                     if len(opk.args) > 1:
                         raise ValueError("window function with more than one argument")
                     for i in range(len(opk.args)):
-                        if not isinstance(
-                            opk.args[0], data_algebra.expr_rep.ColumnReference
-                        ):
+                        if not (isinstance(opk.args[0], data_algebra.expr_rep.ColumnReference)
+                            or isinstance(opk.args[0], data_algebra.expr_rep.Value)):
                             raise ValueError(
-                                "window expression argument must be a column: "
+                                "window expression argument must be a column or value: "
                                 + str(k)
                                 + ": "
                                 + str(opk)
@@ -980,11 +991,11 @@ class ExtendNode(ViewRepresentation):
     def to_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.extend_to_sql(self, using=using, temp_id_source=temp_id_source)
 
-    def eval_implementation(self, *, data_map, eval_env, data_model, narrow):
+    def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
             raise ValueError("Expected data_model to not be None")
         return data_model.extend_step(
-            op=self, data_map=data_map, eval_env=eval_env, narrow=narrow
+            op=self, data_map=data_map, narrow=narrow
         )
 
 
@@ -1028,15 +1039,19 @@ class ProjectNode(ViewRepresentation):
                         + str(opk)
                     )
                 if len(opk.args) > 0:
-                    if not isinstance(
-                        opk.args[0], data_algebra.expr_rep.ColumnReference
-                    ):
+                    if not (isinstance(opk.args[0], data_algebra.expr_rep.ColumnReference)
+                        or isinstance(opk.args[0], data_algebra.expr_rep.Value)):
                         raise ValueError(
-                            "windows expression argument must be a column: "
+                            "windows expression argument must be a column or value: "
                             + str(k)
                             + ": "
                             + str(opk)
                         )
+                if opk.op in data_algebra.expr_rep.fn_names_that_imply_ordered_windowed_situation:
+                    raise ValueError(
+                        str(opk)
+                        + "' is not allowed in project"
+                    )
             else:
                 if not isinstance(opk, data_algebra.user_fn.FnTerm):
                     raise ValueError(
@@ -1113,11 +1128,11 @@ class ProjectNode(ViewRepresentation):
     def to_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.project_to_sql(self, using=using, temp_id_source=temp_id_source)
 
-    def eval_implementation(self, *, data_map, eval_env, data_model, narrow):
+    def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
             raise ValueError("Expected data_model to not be None")
         return data_model.project_step(
-            op=self, data_map=data_map, eval_env=eval_env, narrow=narrow
+            op=self, data_map=data_map, narrow=narrow
         )
 
 
@@ -1180,11 +1195,11 @@ class SelectRowsNode(ViewRepresentation):
             self, using=using, temp_id_source=temp_id_source
         )
 
-    def eval_implementation(self, *, data_map, eval_env, data_model, narrow):
+    def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
             raise ValueError("Expected data_model to not be None")
         return data_model.select_rows_step(
-            op=self, data_map=data_map, eval_env=eval_env, narrow=narrow
+            op=self, data_map=data_map, narrow=narrow
         )
 
 
@@ -1251,11 +1266,11 @@ class SelectColumnsNode(ViewRepresentation):
             self, using=using, temp_id_source=temp_id_source
         )
 
-    def eval_implementation(self, *, data_map, eval_env, data_model, narrow):
+    def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
             raise ValueError("Expected data_model to not be None")
         return data_model.select_columns_step(
-            op=self, data_map=data_map, eval_env=eval_env, narrow=narrow
+            op=self, data_map=data_map, narrow=narrow
         )
 
 
@@ -1322,11 +1337,11 @@ class DropColumnsNode(ViewRepresentation):
             self, using=using, temp_id_source=temp_id_source
         )
 
-    def eval_implementation(self, *, data_map, eval_env, data_model, narrow):
+    def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
             raise ValueError("Expected data_model to not be None")
         return data_model.drop_columns_step(
-            op=self, data_map=data_map, eval_env=eval_env, narrow=narrow
+            op=self, data_map=data_map, narrow=narrow
         )
 
 
@@ -1403,11 +1418,11 @@ class OrderRowsNode(ViewRepresentation):
     def to_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.order_to_sql(self, using=using, temp_id_source=temp_id_source)
 
-    def eval_implementation(self, *, data_map, eval_env, data_model, narrow):
+    def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
             raise ValueError("Expected data_model to not be None")
         return data_model.order_rows_step(
-            op=self, data_map=data_map, eval_env=eval_env, narrow=narrow
+            op=self, data_map=data_map, narrow=narrow
         )
 
     # short-cut main interface
@@ -1498,11 +1513,11 @@ class RenameColumnsNode(ViewRepresentation):
     def to_sql_implementation(self, db_model, *, using, temp_id_source):
         return db_model.rename_to_sql(self, using=using, temp_id_source=temp_id_source)
 
-    def eval_implementation(self, *, data_map, eval_env, data_model, narrow):
+    def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
             raise ValueError("Expected data_model to not be None")
         return data_model.rename_columns_step(
-            op=self, data_map=data_map, eval_env=eval_env, narrow=narrow
+            op=self, data_map=data_map, narrow=narrow
         )
 
 
@@ -1603,11 +1618,11 @@ class NaturalJoinNode(ViewRepresentation):
             self, using=using, temp_id_source=temp_id_source
         )
 
-    def eval_implementation(self, *, data_map, eval_env, data_model, narrow):
+    def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
             raise ValueError("Expected data_model to not be None")
         return data_model.natural_join_step(
-            op=self, data_map=data_map, eval_env=eval_env, narrow=narrow
+            op=self, data_map=data_map, narrow=narrow
         )
 
 
@@ -1703,11 +1718,11 @@ class ConcatRowsNode(ViewRepresentation):
             self, using=using, temp_id_source=temp_id_source
         )
 
-    def eval_implementation(self, *, data_map, eval_env, data_model, narrow):
+    def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
             raise ValueError("Expected data_model to not be None")
         return data_model.concat_rows_step(
-            op=self, data_map=data_map, eval_env=eval_env, narrow=narrow
+            op=self, data_map=data_map, narrow=narrow
         )
 
 
@@ -1806,15 +1821,15 @@ class ConvertRecordsNode(ViewRepresentation):
         near_sql = data_algebra.near_sql.NearSQLq(
             quoted_query_name=db_model.quote_identifier(view_name),
             prev_quoted_query_name=db_model.quote_identifier(prev_view_name),
-            query=query,
+            query=data_algebra.near_sql.NearSQLContainer(sql_text=query),
             terms=terms,
             temp_tables=temp_tables,
         )
         return near_sql
 
-    def eval_implementation(self, *, data_map, eval_env, data_model, narrow):
+    def eval_implementation(self, *, data_map, data_model, narrow):
         if data_model is None:
             raise ValueError("Expected data_model to not be None")
         return data_model.convert_records_step(
-            op=self, data_map=data_map, eval_env=eval_env, narrow=narrow
+            op=self, data_map=data_map, narrow=narrow
         )
