@@ -1,3 +1,7 @@
+
+import data_algebra.cdata
+
+
 class PipeStep:
     def __init__(self):
         pass
@@ -13,12 +17,11 @@ class OperatorPlatform:
         self.node_name = node_name
 
     # noinspection PyPep8Naming
-    def transform(self, X, *, eval_env=None, data_model=None, narrow=True):
+    def transform(self, X, *, data_model=None, narrow=True):
         """
         apply self to data frame X, may or may not commute with composition
 
         :param X: input data frame
-        :param eval_env environment to look for symbols in
         :param data_model implementation to use
         :param narrow logical, if True don't copy unexpected columns
         :return: transformed dataframe
@@ -26,17 +29,16 @@ class OperatorPlatform:
         raise NotImplementedError("base class called")
 
     # noinspection PyPep8Naming
-    def act_on(self, X, *, eval_env=None, data_model=None):
+    def act_on(self, X, *, data_model=None):
         """
         apply self to data frame X, must commute with composition
 
         :param X: input data frame
-        :param eval_env environment to look for symbols in
         :param data_model implementation to use
         :return: transformed dataframe
         """
         return self.transform(
-            X=X, eval_env=eval_env, data_model=data_model, narrow=False
+            X=X, data_model=data_model, narrow=False
         )
 
     def apply_to(self, a, *, target_table_key=None):
@@ -143,6 +145,16 @@ class OperatorPlatform:
     def convert_records(self, record_map, *, temp_namer=None):
         raise NotImplementedError("base class called")
 
+    def map_records(self, blocks_in=None, blocks_out=None, strict=False, temp_namer=None):
+        if (blocks_in is None) and (blocks_out is None):
+            return self  # NO-OP, return source node
+        return self.convert_records(
+            data_algebra.cdata.RecordMap(
+                blocks_in=blocks_in,
+                blocks_out=blocks_out,
+                strict=strict),
+                temp_namer=temp_namer)
+
     # sklearn step style interface
 
     # noinspection PyPep8Naming, PyUnusedLocal
