@@ -117,6 +117,46 @@ def _db_count_expr(dbmodel, expression):
     return 'SUM(1)'
 
 
+def _db_concat_expr(dbmodel, expression):
+    return (
+        "("  # TODO: cast each to char on way in
+        + "|| ".join([dbmodel.expr_to_sql(ai, want_inline_parens=True) for ai in expression.args])
+        + ")"
+    )
+
+
+def _db_coalesce_expr(dbmodel, expression):
+    return (
+        "COALESCE("
+        + ", ".join([dbmodel.expr_to_sql(ai, want_inline_parens=False) for ai in expression.args])
+        + ")"
+    )
+
+
+def _db_round_expr(dbmodel, expression):
+    return (
+        "ROUND("
+        + dbmodel.expr_to_sql(expression.args[0], want_inline_parens=False)
+        + ")"
+    )
+
+
+def _db_floor_expr(dbmodel, expression):
+    return (
+        "FLOOR("
+        + dbmodel.expr_to_sql(expression.args[0], want_inline_parens=False)
+        + ")"
+    )
+
+
+def _db_ceil_expr(dbmodel, expression):
+    return (
+        "CEILING("
+        + dbmodel.expr_to_sql(expression.args[0], want_inline_parens=False)
+        + ")"
+    )
+
+
 db_expr_formatters = {
     "is_null": _db_is_null_expr,
     "is_bad": _db_is_bad_expr,
@@ -125,6 +165,11 @@ db_expr_formatters = {
     "maximum": _db_maximum_expr,
     "minimum": _db_minimum_expr,
     'count': _db_count_expr,
+    'concat': _db_concat_expr,
+    'coalesce': _db_coalesce_expr,
+    'round': _db_round_expr,
+    'floor': _db_floor_expr,
+    'ceil': _db_ceil_expr,
 }
 
 
@@ -380,6 +425,8 @@ class DBModel:
                     # SQL window functions don't like parens
                     return subs[0] + " " + op.upper() + " " + subs[1]
             return op.upper() + "(" + ", ".join(subs) + ")"
+        if isinstance(expression, data_algebra.expr_rep.ListTerm):
+            return self.value_to_sql(expression.value)
         if isinstance(expression, data_algebra.user_fn.FnTerm):
             return expression.to_sql(db_model=self)
         raise TypeError("unexpected type: " + str(type(expression)))
@@ -414,7 +461,7 @@ class DBModel:
                 previous_step_summary=subsql.summary(),
                 terms=terms,  # TODO: implement!
                 quoted_query_name=self.quote_identifier(view_name),
-                sub_sql=subsql.to_sql(columns=using, db_model=self),
+                sub_sql=subsql.to_near_sql(columns=using),
                 temp_tables=subsql.temp_tables.copy(),
             )
         return near_sql
@@ -480,7 +527,7 @@ class DBModel:
             previous_step_summary=subsql.summary(),
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql=subsql.to_sql(columns=subusing, db_model=self),
+            sub_sql=subsql.to_near_sql(columns=subusing),
             temp_tables=subsql.temp_tables.copy(),
         )
         return near_sql
@@ -511,7 +558,7 @@ class DBModel:
             previous_step_summary=subsql.summary(),
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql=subsql.to_sql(columns=subusing, db_model=self),
+            sub_sql=subsql.to_near_sql(columns=subusing),
             suffix=suffix,
             temp_tables=subsql.temp_tables.copy(),
         )
@@ -538,7 +585,7 @@ class DBModel:
             previous_step_summary=subsql.summary(),
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql=subsql.to_sql(columns=subusing, db_model=self),
+            sub_sql=subsql.to_near_sql(columns=subusing),
             suffix=suffix,
             temp_tables=subsql.temp_tables.copy(),
         )
@@ -627,7 +674,7 @@ class DBModel:
             previous_step_summary=subsql.summary(),
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql=subsql.to_sql(columns=subusing, db_model=self),
+            sub_sql=subsql.to_near_sql(columns=subusing),
             suffix=suffix,
             temp_tables=subsql.temp_tables.copy(),
         )
@@ -660,7 +707,7 @@ class DBModel:
             previous_step_summary=subsql.summary(),
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql=subsql.to_sql(columns=subusing, db_model=self),
+            sub_sql=subsql.to_near_sql(columns=subusing),
             temp_tables=subsql.temp_tables.copy(),
         )
         return near_sql
@@ -743,10 +790,10 @@ class DBModel:
         near_sql = data_algebra.near_sql.NearSQLBinaryStep(
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql1=sql_left.to_sql(columns=using_left, db_model=self),
+            sub_sql1=sql_left.to_near_sql(columns=using_left),
             previous_step_summary1=sql_left.summary(),
             joiner=join_node.jointype + " JOIN",
-            sub_sql2=sql_right.to_sql(columns=using_right, db_model=self),
+            sub_sql2=sql_right.to_near_sql(columns=using_right),
             previous_step_summary2=sql_right.summary(),
             suffix=on_terms,
             temp_tables=temp_tables,
@@ -801,16 +848,14 @@ class DBModel:
         near_sql = data_algebra.near_sql.NearSQLUStep(
             terms=terms,
             quoted_query_name=self.quote_identifier(view_name),
-            sub_sql1=sql_left.to_sql(
+            sub_sql1=sql_left.to_near_sql(
                 columns=using_left,
-                db_model=self,
                 force_sql=True,
                 constants=constants_left,
             ),
             previous_step_summary1=sql_left.summary(),
-            sub_sql2=sql_right.to_sql(
+            sub_sql2=sql_right.to_near_sql(
                 columns=using_right,
-                db_model=self,
                 force_sql=True,
                 constants=constants_right,
             ),
@@ -1058,7 +1103,7 @@ class DBHandle(data_algebra.eval_model.EvalModel):
         q_table_name = self.db_model.quote_table_name(table_name)
         self.execute(f'DROP TABLE IF EXISTS {q_table_name}')
 
-    def eval(self, ops, *, data_map=None, result_name=None, eval_env=None, narrow=True):
+    def eval(self, ops, *, data_map=None, result_name=None, narrow=True):
         query = ops.to_sql(self.db_model)
         if result_name is None:
             result_name = self.mk_tmp_name(data_map)
