@@ -1,8 +1,11 @@
 from abc import ABC
 from typing import Union
 
+import numpy
+
+import data_algebra
 import data_algebra.util
-import data_algebra.custom_functions
+
 
 # for some ideas in capturing expressions in Python see:
 #  scipy
@@ -41,10 +44,10 @@ fn_names_that_imply_windowed_situation = {
     "ohlc",
     "pct_change",
     "rank",
-    "sem",
     "shift",
     "size",
     "std",
+    "sum",
     "tail",
     "unique",
     "value_counts",
@@ -52,8 +55,32 @@ fn_names_that_imply_windowed_situation = {
 }
 
 
+fn_names_that_imply_ordered_windowed_situation = {
+    "cumcount",
+    "cummax",
+    "cummin",
+    "cumprod",
+    "cumsum",
+    "row_number",
+}
+
+
+# a competing idea sould be to remove ordering if
+# operator is one of these (instead of forbid)
+fn_names_that_contradict_ordered_windowed_situation = {
+    "count",
+    "max",
+    "min",
+    "prod",
+    "sum",
+}
+
+
 class PreTerm(ABC):
-    # abstract base class, without combination ability
+    """
+    abstract base class, without combination ability
+    """
+
     source_string: Union[str, None]
 
     def __init__(self):
@@ -78,19 +105,19 @@ class PreTerm(ABC):
         """
         pass
 
+    # eval
+
+    def evaluate(self, data_frame):
+        raise NotImplementedError("base class called")
+
     # emitters
 
     def to_python(self, *, want_inline_parens=False):
         raise NotImplementedError("base class method called")  # https://docs.python.org/3/library/exceptions.html
 
-    def to_pandas(self, *, want_inline_parens=False):
-        return self.to_python(want_inline_parens=want_inline_parens)
-
     def to_source(self, *, want_inline_parens=False, dialect="Python"):
         if dialect == "Python":
             return self.to_python(want_inline_parens=want_inline_parens)
-        elif dialect == "Pandas":
-            return self.to_pandas(want_inline_parens=want_inline_parens)
         else:
             raise ValueError("unexpected dialect string: " + str(dialect))
 
@@ -104,10 +131,8 @@ class PreTerm(ABC):
 
 
 class Term(PreTerm, ABC):
-    """Inherit from this class to capture expressions.
-    Abstract class, should be extended for use.
-
-    Used by the parse by lark path via method lookup.
+    """
+    Abstract intermediate class with combination ability
     """
 
     def __init__(self):
@@ -258,6 +283,9 @@ class Term(PreTerm, ABC):
     # TODO: double check https://docs.python.org/3/library/operator.html
     # for more ops such as concat() and so on
 
+    def sign(self):
+        return self.__uop_expr__("sign")
+
     def sin(self):
         return self.__uop_expr__("sin")
 
@@ -348,6 +376,14 @@ class Term(PreTerm, ABC):
     def fmin(self, other):
         return self.__op_expr__("fmin", other, inline=False)
 
+    # more numpy stuff
+    def round(self):
+        return self.__uop_expr__("round")
+
+    def around(self, other):
+        assert isinstance(other, Value)  # digits control
+        return self.__op_expr__("around", other, inline=False)
+
     # pandas style definitions
     # https://pandas.pydata.org/pandas-docs/stable/reference/groupby.html
 
@@ -420,9 +456,6 @@ class Term(PreTerm, ABC):
     def rank(self):
         return self.__uop_expr__("rank")
 
-    def sem(self):
-        return self.__uop_expr__("sem")
-
     def size(self):
         return self.__uop_expr__("size")
 
@@ -457,13 +490,19 @@ class Term(PreTerm, ABC):
     def is_in(self, x):
         return self.__op_expr__("is_in", x, inline=False, method=True)
 
+    def concat(self, x):
+        return self.__op_expr__("concat", x, inline=False, method=True)
+
+    def coalesce(self, x):
+        return self.__op_expr__("coalesce", x, inline=False, method=True)
+
     def co_equalizer(self, x):
         return self.__op_expr__("co_equalizer", x, inline=False, method=True)
 
 
 class Value(Term):
     def __init__(self, value):
-        allowed = [int, float, str, bool, ListTerm, tuple]
+        allowed = [int, float, str, bool]
         if not any([isinstance(value, tp) for tp in allowed]):
             raise TypeError("value type must be one of: " + str(allowed))
         self.value = value
@@ -478,6 +517,9 @@ class Value(Term):
     def replace_view(self, view):
         return self
 
+    def evaluate(self, data_frame):
+        return self.value
+
     def to_python(self, *, want_inline_parens=False):
         return self.value.__repr__()
 
@@ -498,40 +540,12 @@ class UnQuotedStr(str):
         return self.v
 
 
-class FnValue(PreTerm):
-    # represent a function
-    def __init__(self, value, *, name=None):
-        if not callable(value):
-            raise TypeError("value must be callable")
-        if name is None:
-            name = value.__name__
-        self.value = value
-        self.name = name
-        PreTerm.__init__(self)
-
-    def is_equal(self, other):
-        # can't use == as that builds a larger expression
-        if not isinstance(other, FnValue):
-            return False
-        if self.name != other.name:
-            return False
-        return True
-
-    def get_column_names(self, columns_seen):
-        pass
-
-    def replace_view(self, view):
-        pass
-
-    def to_python(self, *, want_inline_parens=False):
-        return UnQuotedStr(self.name)
-
-
 class ListTerm(PreTerm):
+    # derived from PreTerm as this is not combinable
     def __init__(self, value):
-        if not isinstance(value, list):
-            raise TypeError("value type must be a list")
-        self.value = [v for v in value]  # standardize to a list
+        if not isinstance(value, (list, tuple)):
+            raise TypeError("value type must be a list or tuple")
+        self.value = [v for v in value]  # copy and standardize to a list
         PreTerm.__init__(self)
 
     def is_equal(self, other):
@@ -544,6 +558,15 @@ class ListTerm(PreTerm):
         new_list = [ai.replace_view(view) for ai in self.value]
         return ListTerm(new_list)
 
+    def evaluate(self, data_frame):
+        res = [None] * len(self.value)
+        for i in range(len(self.value)):
+            vi = self.value[i]
+            if isinstance(vi, PreTerm):
+                vi = vi.evaluate(data_frame)
+            res[i] = vi
+        return res
+
     def to_python(self, *, want_inline_parens=False):
         def li_to_python(value, *, want_inline_parens=False):
             try:
@@ -562,30 +585,16 @@ class ListTerm(PreTerm):
             + "]"
         )
 
-    def to_pandas(self, *, want_inline_parens=False):
-        return (
-            "["
-            + ", ".join(
-                [
-                    ai.to_pandas(want_inline_parens=want_inline_parens)
-                    for ai in self.value
-                ]
-            )
-            + "]"
-        )
-
     def get_column_names(self, columns_seen):
         for ti in self.value:
             ti.get_column_names(columns_seen)
 
 
 def _enc_value(value):
-    if isinstance(value, Term):
-        return value
-    if isinstance(value, FnValue):
+    if isinstance(value, PreTerm):
         return value
     if callable(value):
-        return FnValue(value)
+        raise ValueError("callable as an argument")
     if isinstance(value, list):
         return ListTerm(value)
     return Value(value)
@@ -607,6 +616,9 @@ class ColumnReference(Term):
                 raise KeyError("column_name '" + str(column_name) + "' must be a column of the given view")
         Term.__init__(self)
 
+    def evaluate(self, data_frame):
+        return data_frame[self.column_name]
+
     def is_equal(self, other):
         # can't use == as that builds a larger expression
         if not isinstance(other, ColumnReference):
@@ -677,39 +689,50 @@ class Expression(Term):
         for a in self.args:
             a.get_column_names(columns_seen)
 
+    def evaluate(self, data_frame):
+        args = [ai.evaluate(data_frame) for ai in self.args]
+        # first check chosen mappings
+        try:
+            return data_algebra.default_data_model.impl_map[self.op](*args)
+        except KeyError:
+            pass
+        # now see if argument (usually Pandas) can do this
+        # doubt we hit in this, as most exposed methods are window methods
+        try:
+            method = getattr(args[0], self.op)
+            if callable(method):
+                return method(*args[1: ])
+        except AttributeError:
+            pass
+        # new see if numpy can do this
+        try:
+            fn = numpy.__dict__[self.op]
+            if callable(fn):
+                return fn(*args)
+        except KeyError:
+            pass
+
     def to_python(self, *, want_inline_parens=False):
         subs = [ai.to_python(want_inline_parens=True) for ai in self.args]
         if len(subs) <= 0:
             return "_" + self.op + "()"
         if len(subs) == 1:
             if self.inline:
-                return self.op + self.args[0].to_pandas(want_inline_parens=True)
-            return subs[0] + "." + self.op + "()"
+                return self.op + self.args[0].to_python(want_inline_parens=True)
+            if isinstance(self.args[0], ColumnReference):
+                return subs[0] + "." + self.op + "()"
+            else:
+                return subs[0] + "." + self.op + "()"
         if len(subs) == 2 and self.inline:
             if want_inline_parens:
                 return "(" + subs[0] + " " + self.op + " " + subs[1] + ")"
             else:
                 return subs[0] + " " + self.op + " " + subs[1]
         if self.method:
-            return subs[0] + "." + self.op + "(" + ", ".join(subs[1:]) + ")"
-        return self.op + "(" + ", ".join(subs) + ")"
-
-    def to_pandas(self, *, want_inline_parens=False):
-        cfmap = data_algebra.default_data_model.custom_function_map
-        if self.op in cfmap.keys():
-            return cfmap[self.op].pandas_formatter(self)
-        if len(self.args) <= 0:
-            return "_" + self.op + "()"
-        if len(self.args) == 1:
-            return (
-                self.op + "(" + self.args[0].to_pandas(want_inline_parens=False) + ")"
-            )
-        subs = [ai.to_pandas(want_inline_parens=True) for ai in self.args]
-        if len(subs) == 2 and self.inline:
-            if want_inline_parens:
-                return "(" + subs[0] + " " + self.op + " " + subs[1] + ")"
+            if isinstance(self.args[0], ColumnReference):
+                return subs[0] + "." + self.op + "(" + ", ".join(subs[1:]) + ")"
             else:
-                return subs[0] + " " + self.op + " " + subs[1]
+                return subs[0] + "." + self.op + "(" + ", ".join(subs[1:]) + ")"
         return self.op + "(" + ", ".join(subs) + ")"
 
 
@@ -736,11 +759,11 @@ def populate_specials(*, column_defs, destination, user_values=None):
     if not isinstance(user_values, dict):
         raise TypeError("user_values should be a dictionary")
     nd = column_defs.copy()
-    # TODO: unify with custom_functions, and more doc!
+    # more doc!
     # makes these symbols available for parsing step
     # need to enter user functions here or as methods on variables
     # Note: a lot of the re-emitters add back the underbar, allow the non
-    # underbar names we see here.  See expr_rep.py Expression to_python and to_pandas.
+    # underbar names we see here.  See expr_rep.py Expression to_python.
     destination["_row_number"] = lambda: data_algebra.expr_rep.Expression(
         op="row_number", args=[]
     )
@@ -749,11 +772,6 @@ def populate_specials(*, column_defs, destination, user_values=None):
     )
     destination["_size"] = lambda: data_algebra.expr_rep.Expression(op="size", args=[])
     destination["connected_components"] = connected_components
-    destination[
-        "partitioned_eval"
-    ] = lambda fn, args, partition: data_algebra.expr_rep.Expression(
-        op="partitioned_eval", args=[fn, args, partition]
-    )
 
 
 def standardize_join_type(join_str):
